import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

public class Main extends MIDlet implements CommandListener {
	private Display display;
	private Displayable nextPage;

	// Game state
	private Arena gameArena;

	
	// Commands applicable to various pages
	private Command doExit;    // Startup page 
	private Command doPause;   // The actual game screen
	private Command doDismiss; // Most other pages

	private int[] colorPal = {
		// The EGA palette
		0x000000,
		/* Let's leave out the dim colours.
		0xAA0000,
		0x00AA00,
		0xAA5500,
		0x0000AA,
		0xAA00AA,
		0x00AAAA,
		0xAAAAAA,
		0x555555,
		*/
		0xFF5555,
		0x55FF55,
		0xFFFF55,
		0x5555FF,
		0xFF55FF,
		0x55FFFF,
		0xFFFFFF
	};

	private int[] bwPal = {
		0x000000,
		0xFFFFFF
	};

	public Main() {
		display = Display.getDisplay(this);

		gameArena = null;
		nextPage = null;

		doExit    = new Command("Exit", Command.EXIT, 1);
		doDismiss = new Command("Dismiss", Command.EXIT, 1);
		doPause   = new Command("Pause", Command.EXIT, 1);
	}

	public void startApp() {
		if (gameArena != null) {
			display.setCurrent(makeGamePage(gameArena));
		} else {
			// Try and load the last game, if we can.
			loadState();

			display.setCurrent(makeStartupPage());
		}
	}

	public void pauseApp() {
		// Nothing.
	}

	public void destroyApp(boolean unconditional) {
		saveState();
	}

	public void commandAction(Command c, Displayable s) {
		if (c == doExit) {
			destroyApp(false);
			notifyDestroyed();

		} else if (c == doDismiss) {
			display.setCurrent(nextPage);
			nextPage = null;

		} else if (c == doPause) {
			nextPage = null;
			display.setCurrent(makeStartupPage());

		} else if (c == List.SELECT_COMMAND) {
			// We assume that the current display item is a List.
			List l = (List) s;
			String label = l.getString(l.getSelectedIndex());

			// Furthermore, we assume the current display is the
			// startup list.
			if (label == "New game") {
				gameArena = new Arena();
				nextPage = null;
				display.setCurrent(makeGamePage(gameArena));

			} else if (label == "Continue game") {
				if (gameArena != null) {
					nextPage = null;
					display.setCurrent(makeGamePage(gameArena));
				} else {
					Alert a = new Alert("Can't continue", 
						"There is no current game to continue",
						null, null);
					a.setTimeout(Alert.FOREVER);

					display.setCurrent(a,
						display.getCurrent());

				}
			} else if (label == "Instructions") {
				Form f = new Form("Instructions");

				f.append(new StringItem("",
					"In this game, you are robot (#). " + 
					"Your job is to find kitten. This " +
					"task is complicated by the " +
					"existence of various things which " +
					"are not kitten. "));

				f.append(new StringItem("",
					"Robot must touch items to " +
					"determine if they are kitten or " +
					"not. The game ends when " +
					"robotfindskitten."));

				f.addCommand(doDismiss);
				f.setCommandListener(this);

				nextPage = display.getCurrent();
				display.setCurrent(f);

			} else if (label == "About") {
				Form f = new Form("About robotfindskitten");
				Image i;

				try {
					i = Image.createImage("/rfklogo.png");
					f.append(new ImageItem("", i,
						ImageItem.LAYOUT_CENTER +
						ImageItem.LAYOUT_NEWLINE_AFTER +
						ImageItem.LAYOUT_NEWLINE_BEFORE,
						"robotfindskitten"));
				} catch (java.io.IOException e) {
					// Nothing.
				}

				f.append(new StringItem("",
					"This version of robotfindskitten " +
					"is a re-implementation of the " +
					"original robotfindskitten for " +
					"Linux."));

				f.append(new StringItem("",
					"For more information, and versions " +
					"of robotfindskitten for other " +
					"platforms (PalmOS, Dreamcast, " +
					"Gameboy Advance, Commodore 64, " +
					"OpenGL...) see the official " +
					"website: " +
					"http://robotfindskitten.org"));

				f.addCommand(doDismiss);
				f.setCommandListener(this);

				nextPage = display.getCurrent();
				display.setCurrent(f);
			}
		}

	}

	public Displayable makeStartupPage() {
		List l = new List("robotfindskitten", List.IMPLICIT);
		if (gameArena != null) {
			l.append("Continue game", null);
		}
		l.append("New game", null);
		l.append("Instructions", null);
		l.append("About", null);

		l.addCommand(doExit);
		l.addCommand(List.SELECT_COMMAND);
		l.setCommandListener(this);

		return l;
	}

	public Displayable makeGamePage(Arena a) {
		int[] pal;
		
		pal = (display.isColor() ? colorPal : bwPal);

		ArenaDisplay ad = 
			new ArenaDisplay(a, pal, this);

		ad.addCommand(doPause);
		ad.setCommandListener(this);

		return ad;
	}

	public void foundNKI(String message) {
		Form f = new Form("Kitten?");

		f.append(new StringItem("", message));
		f.addCommand(doDismiss);
		f.setCommandListener(this);

		nextPage = display.getCurrent();
		display.setCurrent(f);
	}

	public void foundKitten() {
		Form f = new Form("Way to go!");

		try {
			Image i = Image.createImage("/waytogorobot.png");
			f.append(new ImageItem("", i,
					ImageItem.LAYOUT_CENTER +
					ImageItem.LAYOUT_NEWLINE_AFTER +
					ImageItem.LAYOUT_NEWLINE_BEFORE,
					"robotfindskitten"));
		} catch (java.io.IOException e) { }

		f.append(new StringItem("", "Way to go, robot! " +
					"You found kitten!"));

		f.addCommand(doDismiss);
		f.setCommandListener(this);

		gameArena = null;
		nextPage = makeStartupPage();
		display.setCurrent(f);
	}

	private void saveState() {
		javax.microedition.rms.RecordStore rs;
		java.io.ByteArrayOutputStream baos = 
			new java.io.ByteArrayOutputStream();
		java.io.DataOutputStream os = new java.io.DataOutputStream(baos);
		byte[] ba;

		// Clear out the old state.
		try {
			javax.microedition.rms.RecordStore.deleteRecordStore(
					"ArenaState");
		} catch (javax.microedition.rms.RecordStoreException e) {
			// We don't care if it doesn't exist.
		}

		if (gameArena == null) {
			return;
		}

		// Write a new state
		try {
			rs = javax.microedition.rms.RecordStore.openRecordStore(
					"ArenaState", true);

			os.writeLong(gameArena.getSeed());
			os.writeInt(gameArena.getRobotX());
			os.writeInt(gameArena.getRobotY());
			os.flush();
			ba = baos.toByteArray();
			rs.addRecord(ba, 0, ba.length);

			// Tidy everything away.
			rs.closeRecordStore();

		} catch (java.lang.Exception e) {
			System.out.println("Got an exception storing the" +
					"Arena state:" + e.toString());
		}
	}

	private void loadState() {
		javax.microedition.rms.RecordStore rs;
		javax.microedition.rms.RecordEnumeration re;
		java.io.DataInputStream is;

		try {
			// Write a new state
			rs = javax.microedition.rms.RecordStore.openRecordStore(
					"ArenaState", true);

			re = rs.enumerateRecords(null, null, true);
			
			if (!re.hasNextElement()) {
				return;
			}

			is = new java.io.DataInputStream(
					new java.io.ByteArrayInputStream(
						re.nextRecord()
						)
					);

			gameArena = new Arena(is.readLong());
			gameArena.setRobotX(is.readInt());
			gameArena.setRobotY(is.readInt());

			// Tidy everything away.
			rs.closeRecordStore();

		} catch (java.lang.Exception e) {
			gameArena = null;
			System.out.println("Got an exception reading the" +
					"Arena state:" + e.toString());
		}
	}
}
